#!/usr/bin/env python

"""
This module provides an demonstration of how to use the Custodian framework.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyuep@gmail.com"
__date__ = "2/17/13"

import numpy as np

from custodian.custodian import Job, ErrorHandler, Custodian

total = 0
initial = 0


class ExampleJob(Job):
    """
    This example job simply sums a random sequence of 100 numbers between 0
    and 1, adds it to an initial value and puts the value in 'total' global
    variable.
    """

    def __init__(self, jobid):
        self.jobid = jobid

    def setup(self):
        global initial
        initial = 0

    def run(self):
        print "Running job {}".format(self.jobid)
        global initial
        global total
        sequence = np.random.rand(100, 1)
        total = initial + np.sum(sequence)
        print "Current total = {}".format(total)

    def postprocess(self):
        print "Success for job {}".format(self.jobid)

    def name(self):
        return "ExampleJob{}".format(self.jobid)

    @property
    def to_dict(self):
        return {"jobid": self.jobid}

    @staticmethod
    def from_dict(d):
        return ExampleJob(d["jobid"])


class ExampleHandler(ErrorHandler):
    """
    This example error handler checks if the value of total is >= 50. If it is
    not, the Example job should be run again until a total >= 50 is obtained.
    """

    def check(self):
        return total < 50

    def correct(self):
        global initial
        initial += 1
        print "Total < 50. Incrementing initial to {}".format(initial)
        return {"errors": "total < 50", "actions": "increment by 1"}

    @property
    def to_dict(self):
        return {}

    @staticmethod
    def from_dict(d):
        return ExampleHandler()


if __name__ == "__main__":
    c = Custodian([ExampleHandler()], [ExampleJob(i) for i in xrange(10)],
                  max_errors=5)
    c.run()



#!/usr/bin/env python

"""
Created on Jun 1, 2012
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Jun 1, 2012"

import unittest
import os

from custodian.vasp.jobs import VaspJob


test_dir = os.path.join(os.path.dirname(__file__), "..", "..", "..",
                        'test_files')


class VaspJobTest(unittest.TestCase):

    def test_to_from_dict(self):
        v = VaspJob("hello")
        v2 = VaspJob.from_dict(v.to_dict)
        self.assertEqual(type(v2), type(v))
        self.assertEqual(v2.vasp_command, "hello")

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



#!/usr/bin/env python

"""
This module implements the main Custodian class, which manages a list of jobs
given a set of error handlers, and the abstract base classes for the
ErrorHandlers and Jobs.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "May 2, 2012"

import logging
from abc import ABCMeta, abstractmethod, abstractproperty
import json


class Custodian(object):
    """
    The Custodian class is the manager for a list of jobs given a list of
    error handlers. The way it works is as follows:

    1. Let's say you have defined a list of jobs as [job1, job2, job3, ...] and
       you have defined a list of possible error handlers as [err1, err2, ...]
    2. Custodian will run the jobs in the order of job1, job2, ...
    3. At the end of each individual job, Custodian will run through the list
       error handlers. If an error is detected, corrective measures are taken
       and the particular job is rerun.
    """

    def __init__(self, handlers, jobs, max_errors=1):
        """
        Args:
            handlers:
                Error handlers. In order of priority of fixing.
            jobs:
                List of Jobs. Allow for multistep jobs. E.g., give it two
                BasicVaspJobs and you effectively have a aflow
                double-relaxation.
            max_errors:
                Maximum number of errors allowed before exiting.
        """
        self.max_errors = max_errors
        self.jobs = jobs
        self.handlers = handlers

    def run(self):
        """
        Runs the job.

        Returns:
            All errors encountered as a list of list.
            [[error_dicts for job 1], [error_dicts for job 2], ....]
        """
        run_log = []
        total_errors = 0
        for i, job in enumerate(self.jobs):
            run_log.append({"job": job.to_dict, "corrections": []})
            for attempt in xrange(self.max_errors):
                logging.info(
                    "Starting job no. {} ({}) attempt no. {}. Errors thus far"
                    " = {}.".format(i + 1, job.name, attempt + 1,
                                    total_errors))

                # If this is the start of the job, do the setup.
                if not run_log[-1]["corrections"]:
                    job.setup()

                # Run the job.
                job.run()

                # Check for errors using the error handlers and perform
                # corrections.
                error = False
                for h in self.handlers:
                    if h.check():
                        total_errors += 1
                        d = h.correct()
                        logging.error(str(d))
                        run_log[-1]["corrections"].append(d)
                        error = True
                        break

                #Log the corrections to a json file.
                with open("custodian.json", "w") as f:
                    logging.info("Logging to custodian.json...")
                    json.dump(run_log, f, indent=4)

                # If there are no errors detected, perform postprocessing and
                # exit.
                if not error:
                    job.postprocess()
                    break

        if total_errors == self.max_errors:
            logging.info("Max {} errors reached. Exited"
                         .format(self.max_errors))
        else:
            logging.info("Run completed")
        return run_log


class ErrorHandler(object):
    """
    Abstract base class defining the interface for an ErrorHandler.
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def check(self):
        """
        This method is called at the end of a job. Returns a boolean value
        indicating if errors are detected.
        """
        pass

    @abstractmethod
    def correct(self):
        """
        This method is called at the end of a job when an error is detected.
        It should perform any corrective measures relating to the detected
        error.

        This method should return a JSON serializable dict that describes
        the errors and actions taken. E.g.
        {"errors": list_of_errors, "actions": list_of_actions_taken}
        """
        pass

    @abstractproperty
    def to_dict(self):
        """
        This method should return a JSON serializable dict describing the
        ErrorHandler, and can be deserialized using the from_dict static
        method.
        """
        pass

    @staticmethod
    def from_dict(d):
        """
        This simply raises a NotImplementedError to force subclasses to
        implement this static method. Abstract static methods are not
        implemented until Python 3+.
        """
        raise NotImplementedError("ErrorHandler objects must implement a "
                                  "from_dict static method.")


class Job(object):
    """
    Abstract base class defining the interface for a Job.
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def setup(self):
        """
        This method is run before the start of a job. Allows for some
        pre-processing.
        """
        pass

    @abstractmethod
    def run(self):
        """
        This method perform the actual work for the job.
        """
        pass

    @abstractmethod
    def postprocess(self):
        """
        This method is called at the end of a job, *after* error detection.
        This allows post-processing, such as cleanup, analysis of results,
        etc.
        """
        pass

    @abstractproperty
    def name(self):
        """
        A nice string name for the job.
        """
        pass

    @abstractproperty
    def to_dict(self):
        """
        This method should return a JSON serializable dict describing the
        Job, and can be deserialized using the from_dict static
        method.
        """
        pass

    @staticmethod
    def from_dict(d):
        """
        This simply raises a NotImplementedError to force subclasses to
        implement this static method. Abstract static methods are not
        implemented until Python 3+.
        """
        raise NotImplementedError("Job objects must implement a from_dict"
                                  "static method.")


#!/usr/bin/env python

"""
This module implements specific error handlers for VASP runs. These handlers
tries to detect common errors in vasp runs and attempt to fix them on the fly
by modifying the input files.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyuep@gmail.com"
__status__ = "Beta"
__date__ = "2/4/13"

import os
import logging
import tarfile
import glob
import operator

from custodian.custodian import ErrorHandler
from pymatgen.io.vaspio.vasp_input import Poscar, VaspInput
from pymatgen.transformations.standard_transformations import \
    PerturbStructureTransformation
from pymatgen.serializers.json_coders import MSONable

from pymatgen.io.vaspio.vasp_output import Vasprun
from custodian.ansible.intepreter import Modder
from custodian.ansible.actions import FileActions, DictActions


class VaspErrorHandler(ErrorHandler, MSONable):

    error_msgs = {
        "tet": ["Tetrahedron method fails for NKPT<4",
                "Fatal error detecting k-mesh",
                "Fatal error: unable to match k-point",
                "Routine TETIRR needs special values"],
        "inv_rot_mat": ["inverse of rotation matrix was not found (increase "
                        "SYMPREC)"],
        "brmix": ["BRMIX: very serious problems"],
        "subspacematrix": ["WARNING: Sub-Space-Matrix is not hermitian in DAV"],
        "tetirr": ["Routine TETIRR needs special values"],
        "incorrect_shift": ["Could not get correct shifts"],
        "mesh_symmetry": ["Reciprocal lattice and k-lattice belong to "
                          "different class of lattices."]
    }

    def __init__(self, output_filename="vasp.out"):
        self.output_filename = output_filename

    def check(self):
        self.errors = set()
        with open(self.output_filename, "r") as f:
            for line in f:
                l = line.strip()
                for err, msgs in VaspErrorHandler.error_msgs.items():
                    for msg in msgs:
                        if l.startswith(msg):
                            self.errors.add(err)
        return len(self.errors) > 0

    def correct(self):
        backup()
        actions = []
        vi = VaspInput.from_directory(".")

        if "tet" in self.errors:
            actions.append({'dict': 'INCAR',
                            'action': {'_set': {'ISMEAR': 0}}})
        if "inv_rot_mat" in self.errors:
            actions.append({'dict': 'INCAR',
                            'action': {'_set': {'SYMPREC': 1e-8}}})
        if "brmix" in self.errors:
            actions.append({'dict': 'INCAR',
                            'action': {'_set': {'IMIX': 1}}})
        if "subspacematrix" in self.errors:
            actions.append({'dict': 'INCAR',
                            'action': {'_set': {'INCAR->LREAL': False}}})
        if "tetirr" in self.errors or "incorrect_shift" in self.errors:
            actions.append({'dict': 'KPOINTS',
                            'action': {'_set': {'style': "Gamma"}}})
        if "mesh_symmetry" in self.errors:
            m = reduce(operator.mul, vi["KPOINTS"].kpts[0])
            m = max(int(round(m ** (1 / 3))), 1)
            if vi["KPOINTS"].style.lower().startswith("m"):
                m += m % 2
            actions.append({'dict': 'KPOINTS',
                            'action': {'_set': {'kpoints': [[m] * 3]}}})
        m = Modder()
        for a in actions:
            vi[a["dict"]] = m.modify_object(a["action"], vi[a["dict"]])
        vi["INCAR"].write_file("INCAR")
        vi["POSCAR"].write_file("POSCAR")
        vi["KPOINTS"].write_file("KPOINTS")
        return {"errors": list(self.errors), "actions": actions}

    def __str__(self):
        return "Vasp error"

    @property
    def to_dict(self):
        return {"@module": self.__class__.__module__,
                "@class": self.__class__.__name__,
                "output_filename": self.output_filename}

    @staticmethod
    def from_dict(d):
        return VaspErrorHandler(d["output_filename"])


class UnconvergedErrorHandler(ErrorHandler, MSONable):
    """
    Check if a run is converged
    """
    def __init__(self, output_filename="vasprun.xml"):
        self.output_filename = output_filename

    def check(self):
        try:
            v = Vasprun(self.output_filename)
            if not v.converged:
                return True
        except:
            return True

    def correct(self):
        backup()
        actions = [{'file': 'CONTCAR',
                    'action': {'_file_copy': {'dest': 'POSCAR'}}},
                   {'dict': 'INCAR',
                    'action': {'_set': {'ISTART': 1}}}]
        vi = VaspInput.from_directory(".")
        m = Modder(actions=[DictActions, FileActions])
        for a in actions:
            if "dict" in a:
                vi[a["dict"]] = m.modify_object(a["action"], vi[a["dict"]])
            elif "file" in a:
                m.modify(a["action"], a["file"])
        vi["INCAR"].write_file("INCAR")

        return {"errors": ["Unconverged"], "actions": actions}

    def __str__(self):
        return "Run unconverged."

    @property
    def to_dict(self):
        return {"@module": self.__class__.__module__,
                "@class": self.__class__.__name__,
                "output_filename": self.output_filename}

    @staticmethod
    def from_dict(d):
        return UnconvergedErrorHandler(d["output_filename"])


class PoscarErrorHandler(ErrorHandler, MSONable):

    def __init__(self, output_filename="vasp.out"):
        self.output_filename = output_filename

    def check(self):
        with open(self.output_filename, "r") as f:
            output = f.read()
            for line in output.split("\n"):
                l = line.strip()
                if l.startswith("Found some non-integer element in rotation "
                                "matrix"):
                    return True
        return False

    def correct(self):
        backup()
        p = Poscar.from_file("POSCAR")
        s = p.structure
        trans = PerturbStructureTransformation(0.05)
        new_s = trans.apply_transformation(s)
        actions = [{'dict': 'POSCAR',
                    'action': {'_set': {'structure': new_s.to_dict}}}]
        m = Modder()
        vi = VaspInput.from_directory(".")
        for a in actions:
            vi[a["dict"]] = m.modify_object(a["action"], vi[a["dict"]])
        vi["POSCAR"].write_file("POSCAR")

        return {"errors": ["Rotation matrix"],
                "actions": actions}

    @property
    def to_dict(self):
        return {"@module": self.__class__.__module__,
                "@class": self.__class__.__name__,
                "output_filename": self.output_filename}

    @staticmethod
    def from_dict(d):
        return PoscarErrorHandler(d["output_filename"])


def backup():
    error_num = 0
    for f in glob.glob("error.*.tar.gz"):
        toks = f.split(".")
        error_num = max(error_num, int(toks[1]))
    filename = "error.{}.tar.gz".format(error_num + 1)
    logging.info("Backing up run to {}.".format(filename))
    tar = tarfile.open(filename, "w:gz")
    for f in os.listdir("."):
        if not (f.startswith("error") and f.endswith(".tar.gz")):
            tar.add(f)
    tar.close()



#!/usr/bin/env python

"""
This module implements basic kinds of jobs for VASP runs.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyuep@gmail.com"
__status__ = "Beta"
__date__ = "2/4/13"


import subprocess
import os
import shutil

from pymatgen.util.io_utils import zopen
from pymatgen.io.vaspio.vasp_input import VaspInput
from pymatgen.io.smartio import read_structure
from pymatgen.io.vaspio_set import MITVaspInputSet
from pymatgen.serializers.json_coders import MSONable, PMGJSONDecoder

from custodian.ansible.intepreter import Modder
from custodian.ansible.actions import FileActions, DictActions
from custodian.custodian import Job


VASP_INPUT_FILES = {"INCAR", "POSCAR", "POTCAR", "KPOINTS"}

VASP_OUTPUT_FILES = ['DOSCAR', 'INCAR', 'KPOINTS', 'POSCAR', 'PROCAR',
                     'vasprun.xml', 'CHGCAR', 'CHG', 'EIGENVAL', 'OSZICAR',
                     'WAVECAR', 'CONTCAR', 'IBZKPT', 'OUTCAR']


class VaspJob(Job, MSONable):
    """
    A basic vasp job. Just runs whatever is in the directory. But conceivably
     can be a complex processing of inputs etc. with initialization.
    """

    def __init__(self, vasp_command, output_file="vasp.out", suffix="",
                 final=True, gzipped=False, backup=True,
                 default_vasp_input_set=MITVaspInputSet(),
                 settings_override=None):
        """
        This constructor is necessarily complex due to the need for
        flexibility. For standard kinds of runs, it's often better to use one
        of the static constructors.

        Args:
            vasp_command:
                Command to run vasp as a list of args. For example,
                if you are using mpirun, it can be something like
                ["mpirun", "pvasp.5.2.11"]
            output_file:
                Name of file to direct standard out to. Defaults to vasp.out.
            suffix:
                A suffix to be appended to the final output. E.g.,
                to rename all VASP output from say vasp.out to
                vasp.out.relax1, provide ".relax1" as the suffix.
            final:
                Boolean indicating whether this is the final vasp job in a
                series. Defaults to True.
            backup:
                Boolean whether to backup the initial input files. If True,
                the INCAR, KPOINTS, POSCAR and POTCAR will be copied with a
                ".orig" appended. Defaults to True.
            gzipped:
                Whether to gzip the final output. Defaults to False.
            default_vasp_input_set:
                Species the default input set to use for directories that do
                not contain full set of VASP input files. For example,
                if a directory contains only a POSCAR or a cif,
                the vasp input set will be used to generate the necessary
                input files for the run. If the directory already
                contain a full set of VASP input files,
                this input is ignored. Defaults to the MITVaspInputSet.
            settings_override:
                An ansible style list of dict to override changes. For example,
                to set ISTART=1 for subsequent runs and to copy the CONTCAR
                to the POSCAR, you will provide::

                    [{"dict": "INCAR", "action": {"_set": {"ISTART": 1}}},
                     {"filename": "CONTCAR",
                      "action": {"_file_copy": {"dest": "POSCAR"}}}]
        """
        self.vasp_command = vasp_command
        self.output_file = output_file
        self.final = final
        self.backup = backup
        self.gzipped = gzipped
        self.default_vis = default_vasp_input_set
        self.suffix = suffix
        self.settings_override = settings_override

    def setup(self):
        files = os.listdir(".")
        num_structures = 0
        if not set(files).issuperset(VASP_INPUT_FILES):
            for f in files:
                try:
                    struct = read_structure(f)
                    num_structures += 1
                except:
                    pass
            if num_structures != 1:
                raise RuntimeError("{} structures found. Unable to continue."
                                   .format(num_structures))
            else:
                self.default_vis.write_input(struct, ".")

        if self.backup:
            for f in VASP_INPUT_FILES:
                shutil.copy(f, "{}.orig".format(f))

        if self.settings_override is not None:
            vi = VaspInput.from_directory(".")
            m = Modder([FileActions, DictActions])
            modified = []
            for a in self.settings_override:
                if "dict" in a:
                    modified.append(a["dict"])
                    vi[a["dict"]] = m.modify_object(a["action"],
                                                    vi[a["dict"]])
                elif "filename" in a:
                    m.modify(a["action"], a["filename"])
            for f in modified:
                vi[f].write_file(f)

    def run(self):
        with open(self.output_file, 'w') as f:
            subprocess.call(self.vasp_command, stdout=f)

    def postprocess(self):
        for f in VASP_OUTPUT_FILES + [self.output_file]:
            if os.path.exists(f):
                if self.final and self.suffix != "":
                    shutil.move(f, "{}{}".format(f, self.suffix))
                elif self.suffix != "":
                    shutil.copy(f, "{}{}".format(f, self.suffix))

        if self.gzipped:
            gzip_directory(".")

    @property
    def name(self):
        return "Vasp Job"

    @staticmethod
    def double_relaxation_run(vasp_command, gzipped=True):
        """
        Returns a list of two jobs corresponding to an AFLOW style double
        relaxation run.

        Args:
            vasp_command:
                Command to run vasp as a list of args. For example,
                if you are using mpirun, it can be something like
                ["mpirun", "pvasp.5.2.11"]

        Returns:
            List of two jobs corresponding to an AFLOW style run.
        """
        return [VaspJob(vasp_command, final=False, suffix=".relax1"),
                VaspJob(
                    vasp_command, final=True, backup=False,
                    suffix=".relax2", gzipped=gzipped,
                    settings_override=[
                        {"dict": "INCAR",
                         "action": {"_set": {"ISTART": 1}}},
                        {"filename": "CONTCAR",
                         "action": {"_file_copy": {"dest": "POSCAR"}}}])]

    @property
    def to_dict(self):
        d = dict(vasp_command=self.vasp_command,
                 output_file=self.output_file, suffix=self.suffix,
                 final=self.final, gzipped=self.gzipped, backup=self.backup,
                 default_vasp_input_set=self.default_vis.to_dict,
                 settings_override=self.settings_override
                 )
        d["@module"] = self.__class__.__module__
        d["@class"] = self.__class__.__name__
        return d

    @staticmethod
    def from_dict(d):
        vis = PMGJSONDecoder().process_decoded(d["default_vasp_input_set"])
        return VaspJob(
            vasp_command=d["vasp_command"], output_file=d["output_file"],
            suffix=d["suffix"], final=d["final"], gzipped=d["gzipped"],
            backup=d["backup"], default_vasp_input_set=vis,
            settings_override=d["settings_override"])


def gzip_directory(path):
    """
    Gzips all files in a directory.

    Args:
        path:
            Path to directory.
    """
    for f in os.listdir(path):
        if not f.endswith("gz"):
            with zopen(f, 'rb') as f_in, \
                    zopen('{}.gz'.format(f), 'wb') as f_out:
                f_out.writelines(f_in)
            os.remove(f)


#!/usr/bin/env python

"""
Created on Jun 1, 2012
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Jun 1, 2012"

import unittest
import os
import shutil

from custodian.vasp.handlers import VaspErrorHandler, \
    UnconvergedErrorHandler, PoscarErrorHandler


test_dir = os.path.join(os.path.dirname(__file__), "..", "..", "..",
                        'test_files')


class VaspErrorHandlerTest(unittest.TestCase):

    def test_check_correct(self):
        if "VASP_PSP_DIR" not in os.environ:
            os.environ["VASP_PSP_DIR"] = test_dir
        os.chdir(test_dir)
        shutil.copy("INCAR", "INCAR.orig")
        shutil.copy("KPOINTS", "KPOINTS.orig")
        h = VaspErrorHandler("vasp.teterror")
        h.check()
        d = h.correct()
        self.assertEqual(d["errors"], ['tet'])
        self.assertEqual(d["actions"],
                         [{'action': {'_set': {'ISMEAR': 0}},
                           'dict': 'INCAR'}])
        h = VaspErrorHandler("vasp.classrotmat")
        h.check()
        d = h.correct()
        self.assertEqual(d["errors"], ['mesh_symmetry'])
        self.assertEqual(d["actions"],
                         [{'action': {'_set': {'kpoints': [[4, 4, 4]]}},
                           'dict': 'KPOINTS'}])
        shutil.move("INCAR.orig", "INCAR")
        shutil.move("KPOINTS.orig", "KPOINTS")
        os.remove(os.path.join(test_dir, "error.1.tar.gz"))
        os.remove(os.path.join(test_dir, "error.2.tar.gz"))


    def test_to_from_dict(self):
        h = VaspErrorHandler("random_name")
        h2 = VaspErrorHandler.from_dict(h.to_dict)
        self.assertEqual(type(h2), type(h))
        self.assertEqual(h2.output_filename, "random_name")


class UnconvergedErrorHandlerTest(unittest.TestCase):

    def test_check_correct(self):
        if "VASP_PSP_DIR" not in os.environ:
            os.environ["VASP_PSP_DIR"] = test_dir
        subdir = os.path.join(test_dir, "unconverged")
        os.chdir(subdir)
        h = UnconvergedErrorHandler("POTCAR")
        h.check()
        d = h.correct()
        self.assertEqual(d["errors"], ['Unconverged'])
        self.assertEqual(d["actions"],
                         [{'file': 'CONTCAR',
                           'action': {'_file_copy': {'dest': 'POSCAR'}}},
                          {'dict': 'INCAR',
                           'action': {'_set': {'ISTART': 1}}}])
        os.remove(os.path.join(subdir, "error.1.tar.gz"))

    def test_to_from_dict(self):
        h = UnconvergedErrorHandler("random_name.xml")
        h2 = UnconvergedErrorHandler.from_dict(h.to_dict)
        self.assertEqual(type(h2), UnconvergedErrorHandler)
        self.assertEqual(h2.output_filename, "random_name.xml")


class PoscarErrorHandlerTest(unittest.TestCase):

    def test_check_correct(self):
        if "VASP_PSP_DIR" not in os.environ:
            os.environ["VASP_PSP_DIR"] = test_dir
        subdir = os.path.join(test_dir, "poscar_error")
        os.chdir(subdir)
        shutil.copy("POSCAR", "POSCAR.orig")
        h = PoscarErrorHandler()
        h.check()
        d = h.correct()
        self.assertEqual(d["errors"], ["Rotation matrix"])
        os.remove(os.path.join(subdir, "error.1.tar.gz"))
        shutil.copy("POSCAR.orig", "POSCAR")
        os.remove("POSCAR.orig")

    def test_to_from_dict(self):
        h = PoscarErrorHandler("random_name.out")
        h2 = PoscarErrorHandler.from_dict(h.to_dict)
        self.assertEqual(type(h2), PoscarErrorHandler)
        self.assertEqual(h2.output_filename, "random_name.out")


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



#!/usr/bin/env python

'''
Created on Jun 1, 2012
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Jun 1, 2012"

import unittest

from custodian.ansible.intepreter import Modder
from custodian.ansible.actions import FileActions


class ModderTest(unittest.TestCase):

    def test_dict_modify(self):
        modder = Modder()
        d = {"Hello": "World"}
        mod = {'_set': {'Hello': 'Universe', 'Bye': 'World'}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'Hello': 'Universe'})
        mod = {'_unset': {'Hello': 1}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World'})
        mod = {'_push': {'List': 1}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [1]})
        mod = {'_push': {'List': 2}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [1, 2]})
        mod = {'_inc': {'num': 5}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [1, 2], 'num': 5})
        mod = {'_inc': {'num': 5}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [1, 2], 'num': 10})
        mod = {'_rename': {'num': 'number'}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [1, 2], 'number': 10})
        mod = {'_add_to_set': {'List': 2}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [1, 2], 'number': 10})
        mod = {'_add_to_set': {'List': 3}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [1, 2, 3], 'number': 10})
        mod = {'_add_to_set': {'number': 3}}
        self.assertRaises(ValueError, modder.modify, mod, d)
        mod = {'_pull': {'List': 1}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [2, 3], 'number': 10})
        mod = {'_pull_all': {'List': [2, 3]}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [], 'number': 10})
        mod = {'_push_all': {'List': xrange(10)}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World',
                             'List': [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                             'number': 10})
        mod = {'_pop': {'List': 1}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World',
                             'List': [0, 1, 2, 3, 4, 5, 6, 7, 8],
                             'number': 10})
        mod = {'_pop': {'List':-1}}
        modder.modify(mod, d)
        self.assertEqual(d, {'Bye': 'World', 'List': [1, 2, 3, 4, 5, 6, 7, 8],
                             'number': 10})
        d = {}
        mod = {'_set': {'a->b->c': 100}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 100}}})
        mod = {'_set': {'a->b->d': 200}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 100, 'd': 200}}})
        mod = {'_set': {'a->b->d': 300}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 100, 'd': 300}}})
        mod = {'_unset': {'a->b->d': 300}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 100}}})
        mod = {'_push': {'a->e->f': 300}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 100}, 'e': {'f': [300]}}})
        mod = {'_push_all': {'a->e->f': [100, 200]}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 100},
                                   'e': {'f': [300, 100, 200]}}})
        mod = {'_inc': {'a->b->c': 2}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 102},
                                   'e': {'f': [300, 100, 200]}}})
        mod = {'_pull': {'a->e->f': 300}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 102}, 'e': {'f': [100, 200]}}})
        mod = {'_pull_all': {'a->e->f': [100, 200]}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 102}, 'e': {'f': []}}})
        mod = {'_push_all': {'a->e->f': [101, 201, 301, 401]}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 102},
                                   'e': {'f': [101, 201, 301, 401]}}})
        mod = {'_pop': {'a->e->f': 1}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 102},
                                   'e': {'f': [101, 201, 301]}}})
        mod = {'_pop': {'a->e->f':-1}}
        modder.modify(mod, d)
        self.assertEqual(d, {'a': {'b': {'c': 102}, 'e': {'f': [201, 301]}}})

    def test_file_modify(self):
        modder = Modder(actions=[FileActions])
        modder.modify({'_file_create': {'content': 'Test data'}}, 'test_file')
        modder.modify({'_file_copy': {'dest': 'test_file_copy'}}, 'test_file')
        modder.modify({'_file_copy': {'dest1': 'test_file_copy1',
                                      'dest2': 'test_file_copy2'}},
                      'test_file')
        modder.modify({'_file_move': {'dest': 'renamed_test_file'}},
                      'test_file')
        modder.modify({'_file_delete': {'mode': "actual"}},
                      'renamed_test_file')
        modder.modify({'_file_delete': {'mode': "actual"}}, 'test_file_copy')
        modder.modify({'_file_delete': {'mode': "actual"}}, 'test_file_copy1')
        modder.modify({'_file_delete': {'mode': "actual"}}, 'test_file_copy2')

    def test_strict_mode(self):
        modder = Modder(actions=[FileActions])
        d = {"Hello": "World"}
        mod = {'_set': {'Hello': 'Universe', 'Bye': 'World'}}
        self.assertRaises(ValueError, modder.modify, mod, d)

        #In non-strict mode, unknown actions are ignored.
        d = {"Hello": "World"}
        modder = Modder(actions=[FileActions], strict=False)
        modder.modify(mod, d)
        self.assertEqual(d, {"Hello": "World"})

        #File actions not supported
        modder = Modder()
        self.assertRaises(ValueError, modder.modify,
                          {'_file_create': {'content': 'Test data'}},
                          'test_file')

    def test_modify_object(self):
        modder = Modder()
        o = MyObject(1)
        self.assertEqual(o.b["a"], 1)
        mod_o = modder.modify_object({'_set': {'b->a': 20}}, o)
        self.assertEqual(mod_o.b["a"], 20)

class MyObject():

    def __init__(self, a):
        self.b = {'a': a}

    @property
    def to_dict(self):
        return {'b': {'a': self.b['a']}}

    @staticmethod
    def from_dict(d):
        return MyObject(d["b"]["a"])

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()
